@@ -6,7 +6,7 @@ from ejcli.error import EJError
 def _http_header_capitalize(h):
     return '-'.join(i[:1].upper()+i[1:].lower() for i in h.split('-'))
 
-def do_http(url, method, headers, data=b''):
+def do_http(url, method, headers={}, data=b''):
     if '://' not in url:
         raise EJError("Invalid URL")
     proto, path = url.split('://', 1)
@@ -123,7 +123,7 @@ class Ejudge(Backend):
         base_url = rhd['Location'].split('&')[0]
         if 'new-client?SID=' in base_url:
             urls = ej371.get_urls(base_url)
-        elif '/user/' in base_url or '/client/' in base_url or '/register?SID=' in base_url:
+        elif any(i in base_url for i in ('/user/', '/client/', '/register/', '/register?SID=')):
             urls = ej373.get_urls(base_url)
         else:
             raise EJError("Unknown ejudge version.")
@@ -205,7 +205,9 @@ class Ejudge(Backend):
             else: break
     #   x = '-----------------------------850577185583170701784494929'
         data = b'\r\n'.join(b'--'+x+b'\r\nContent-Disposition: form-data; name='+i for i in data)+b'\r\n--'+x+b'--\r\n'
-        return post(url, data, {'Content-Type': 'multipart/form-data; boundary='+x.decode('ascii'), 'Cookie': self.cookie})
+        ans = post(url, data, {'Content-Type': 'multipart/form-data; boundary='+x.decode('ascii'), 'Cookie': self.cookie})
+        with self.cache_lock: self.stop_caching()
+        return ans
     def status(self):
         code, headers, data = self._cache_get(self.urls['summary'])
         if code != 200:
@@ -349,6 +351,7 @@ class Ejudge(Backend):
     def submit_clar(self, task, subject, text): 
         if post(self.urls['submit'], {'SID': self.urls['sid'], 'prob_id': task, 'subject': subject, 'text': text, 'action_41': 'Send!'}, {'Cookie': self.cookie})[0] != 302:
             raise EJError("Failed to submit clar")
+        with self.cache_lock: self.stop_caching()
     def read_clar(self, id):
         code, headers, data = self._cache_get(self.urls['read_clar'].format(clar_id=id))
         data = html.unescape(data.decode('utf-8').split('<pre class="message">', 1)[1].split('</pre>', 1)[0])
@@ -415,3 +418,15 @@ class Ejudge(Backend):
         return ans
     def stop_caching(self):
         self._get_cache.clear()
+    def contest_list(self):
+        if isinstance(self, str): url = self
+        else: url = self.urls['contest_list']
+        code, headers, data = get(url)
+        if code != 200:
+            return []
+        ans = []
+        for i in data.decode('utf-8').split('<td><a href="')[1:]:
+            url = html.unescape(i.split('"', 1)[0])
+            name = html.unescape(i.split('>', 1)[1].split('<', 1)[0])
+            ans.append((name, url, {}))
+        return ans
